/*
* This file is part of fwmon
* Copyright (c) 2001,2002 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char rcsid[]=
	"@(#) $Id: libpcap.c,v 1.17 2002/02/11 10:30:19 gianni Exp $";

#include "g_globals.h"
#include "libpcap.h"

#include <sys/stat.h>
#include <sys/uio.h>
#include <pcap.h>

int cap_fd=-1;

int lpcap_init(void)
{
	struct pcap_file_header phdr;
	int fe=0;
	struct stat st;

	if ( !flag(FLAG_LIBPCAP) ) return 1;

	if ( (cap_fd=open(globals.pcapfile, O_RDWR | O_CREAT, 00600))<0 ) {
		perror("open() tcpdump file");
		return 0;
	}

	if ( fstat(cap_fd, &st) ) {
		perror("fstat() tcpdump file");
		close(cap_fd);
		return 0;
	}

	/* Check if it is a valid pcap file, if not
	* clobber it */
	if ( st.st_size > sizeof(phdr) ) {
		/* Read the header */
		if ( read(cap_fd, &phdr, sizeof(phdr)) < sizeof(phdr)) {
			perror("read() tcpdump header");
			close(cap_fd);
			return 0;
		}

		/* Check the header */
		if ( ((phdr.magic == 0xa1b2c3d4) && 
			(phdr.linktype == DLT_RAW)) ) {
			lseek(cap_fd, 0, SEEK_END);
			fe=1;
		}
	}

	if ( fe==0 && st.st_size ) {
		print_err("warning: clobbering %s" CRLF, globals.pcapfile);
		ftruncate(cap_fd, 0);
	}
	
	/* Setup the header if the file didnt
	 * already exist. */
	if ( !fe ) {
		//phdr.magic = 0xa1b2c3d4;
		phdr.magic = 0xa1b2cd34;
		phdr.version_major = PCAP_VERSION_MAJOR;
		phdr.version_minor = PCAP_VERSION_MINOR;
		phdr.thiszone = 0;
		phdr.sigfigs = 0;
		phdr.snaplen = BUFFER_SIZE;
		phdr.linktype = DLT_RAW;

		if ( write(cap_fd, &phdr, sizeof(phdr)) < 0 ) {
			perror("writing pcap header");
			return 0;
		}
		
		fsync(cap_fd);
	}

	return 1;
}

void lpcap_close(void)
{
	if ( !flag(FLAG_LIBPCAP) ) return;
	if ( cap_fd >= 0 ) {
		close(cap_fd);
		cap_fd = -1;
	}
}

void lpcap_dump ( p_packet p )
{
	static struct pcap_pkthdr phdr;
	static struct iovec iov[2]={
		{&phdr, sizeof(phdr)},
		{NULL, 0}
	};

	/* Setup header vector */
	phdr.caplen	= p->hdr->len;
	phdr.len 	= ntohs(p->ip->tot_len);
	phdr.ts.tv_sec	= p->ts.tv_sec;
	phdr.ts.tv_usec	= p->ts.tv_usec;

	/* Setup the data vector */
	iov[1].iov_base=p->ip;
	iov[1].iov_len=phdr.caplen;

	/* write both vectors at once */
	writev(cap_fd, iov, 2);
}
