/*
* This file is part of fwmon
* Copyright (c) 2001,2002 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char rcsid[]=
	"@(#) $Id: print.c,v 1.19 2002/02/04 08:35:03 gianni Exp $";

#include "g_globals.h"
#include "print.h"
#include "util.h"
#include <time.h>
#include <ctype.h>

char prnbuf[BUFFER_SIZE];
char *b;
int log_fd=-1;
int print_init(void)
{
	if ( flag(FLAG_STDOUT) && flag(FLAG_DAEMON) ){
		flag_unset(FLAG_STDOUT);
	}
	
	if ( flag(FLAG_LOGFILE) ){
		if ( !globals.logfile || !*(globals.logfile) ) {
			print_err(ERR_OPEN_LOG CRLF);
			return 0;
		}
		
		if ( (log_fd=open(globals.logfile,
			O_WRONLY | O_APPEND | O_CREAT, 00600))<0 ) {
			perror(ERR_OPEN_LOG);
			return 0;
		}
	}

	return 1;
}

void print_close(void)
{
	if ( flag(FLAG_LOGFILE) ){
		if ( log_fd >= 0 ) {
			close(log_fd);
			log_fd=-1;
		}
	}
}

void print_dump ( p_packet p )
{
	int sport=0, dport=0;
	static char sip[32], dip[32];
	char tb[512];
	struct tm *the_time;

	/* basic header types */
	struct tcphdr  *tcp = p->trans;
	struct udphdr  *udp = p->trans;
	struct icmphdr *icmp = p->trans;

	b=prnbuf;
	format_ip( p->ip->saddr, sip);
	format_ip( p->ip->daddr, dip);

	/* set ports, dependant on protocol */
	switch ( p->ip->protocol ){
		case IPPROTO_TCP:
			sport= ntohs(tcp->source);
			dport= ntohs(tcp->dest);
			break;
		case IPPROTO_UDP:
			sport=ntohs(udp->source);
			dport=ntohs(udp->dest);
			break;
		case IPPROTO_ICMP:
			sport=icmp->type;
			dport=icmp->code;
			break;
	}

	if ( ! (the_time=localtime((time_t *)&p->ts.tv_sec)) ) {
		return;
	}
	
	if ( flag(FLAG_SQL) ) {
		strftime(tb, 512-1, "%Y-%m-%d %H:%M:%S+01", the_time);
		b+=sprintf(b, "INSERT INTO \"ip_quads\" (\"host\", \"ts\", "
			"\"proto\", \"src_ip\", \"dst_ip\", \"src_port\", "
			"\"dst_port\") VALUES ('%s', '%s', %i, '%s', "
			"'%s', %i, %i);" CRLF, globals.uname,
			tb, p->ip->protocol, sip,
			dip, sport, dport);
		print_buffer();
		return;
	}
	
	strftime(tb, 512-1, flag(FLAG_SHOWDATE) ? "%d-%m-%Y %H:%M:%S" :
		"%H:%M:%S", the_time);

	/* date/info */
	b+=sprintf(b, "%s [%s/%i] %s %s:%i -> %s:%i", tb,
			p->hdr->iface, p->hdr->mark,
		       	get_proto(p->ip->protocol),
			sip, sport, dip, dport);

	/* print proto specific stuff */
	switch ( p->ip->protocol ){
		case IPPROTO_TCP:
			b+=sprintf(b, " %s", get_tcpflags(tcp));
			break;
		case IPPROTO_ICMP:
			b+=sprintf(b, " %s", icmp_type(icmp->type, icmp->code));
			break;
	}
	b+=sprintf(b, " ttl=%i len=%d" CRLF, p->ip->ttl, ntohs(p->ip->tot_len));

	/* Print the hex data */
	if ( !flag(FLAG_NODATA) ) {
		b=hex_dump( p );
	}

	print_buffer();
}

void print_buffer(void)
{
	int len=strlen((char *)&prnbuf);
	
	/* Write to stdout */
	if ( flag(FLAG_STDOUT) && !flag(FLAG_DAEMON) ) {
		write(1, prnbuf, len);
	}

	/* Write to logfile */
	if ( flag(FLAG_LOGFILE) && log_fd >= 0 ){
		write(log_fd, prnbuf, len);
	}

	b=prnbuf;
}

char *hex_dump( p_packet p )
{
	unsigned int i,j=0;
	unsigned int line;
	unsigned int plen=ntohs(p->ip->tot_len);
	unsigned char *tmp;

	tmp=(unsigned char*)p->ip;
	
	if ( plen > p->hdr->len ) plen=p->hdr->len;
	
	while ( j < plen ) {
		/* work out length of line */
		if ( (j+globals.length) > plen )
			line = plen-j;
		else
			line = globals.length;

		/* print offset */
		b+=sprintf(b, "%04X : ", j);

		if ( !flag(FLAG_NOASCII) ) {	
			for ( i=0; i<globals.length; i++ ) {
				if ( i<line ) {
					if ( isprint(tmp[i]) )
						b+=sprintf(b, "%c", tmp[i]);
					else
						b+=sprintf(b, ".");
				}else{
					b+=sprintf(b, " ");
				}
			}
		}
		
		if ( !flag(FLAG_NOHEX) ) {
			for ( i=0; i<line; i++ ) {
					b+=sprintf(b, " %02X", tmp[i]);
			}
		}

		j+=line;
		tmp+=line;
		b+=sprintf(b, CRLF);

		/* AUDIT: Flush buffer when we are getting close to filling it */
		if ( (unsigned long)b >= 
			((unsigned long)(&prnbuf)+BUFFER_SIZE-((line*4)+16)) ) {
			print_buffer();
		}
	}

	b+=sprintf(b, CRLF);
	return b;
}
